@@ -22,15 +22,19 @@ from ndg.oauth.server.lib.access_token.bearer_token_generator import \
     BearerTokenGenerator
 from ndg.oauth.server.lib.access_token.myproxy_cert_token_generator \
     import MyProxyCertTokenGenerator
-from ndg.oauth.server.lib.authenticate.certificate_client_authenticator \
-    import CertificateClientAuthenticator
-from ndg.oauth.server.lib.authenticate.noop_client_authenticator import \
-    NoopClientAuthenticator
+from ndg.oauth.server.lib.authenticate.certificate_authenticator \
+    import CertificateAuthenticator
+from ndg.oauth.server.lib.authenticate.noop_authenticator import \
+    NoopAuthenticator
+from ndg.oauth.server.lib.authenticate.password_authenticator \
+    import PasswordAuthenticator
 from ndg.oauth.server.lib.authorization_server import AuthorizationServer
 from ndg.oauth.server.lib.authorize.authorizer_storing_identifier import \
     AuthorizerStoringIdentifier
 from ndg.oauth.server.lib.resource_request.myproxy_cert_request import \
     MyproxyCertRequest
+from ndg.oauth.server.lib.register.client import ClientRegister
+from ndg.oauth.server.lib.register.resource import ResourceRegister
 
 log = logging.getLogger(__name__)
 
@@ -61,6 +65,8 @@ class Oauth2ServerMiddleware(object):
     CLIENT_AUTHORIZATION_URL_OPTION = 'client_authorization_url'
     CLIENT_AUTHORIZATIONS_KEY_OPTION = 'client_authorizations_key'
     CLIENT_REGISTER_OPTION = 'client_register'
+    RESOURCE_AUTHENTICATION_METHOD_OPTION = 'resource_authentication_method'
+    RESOURCE_REGISTER_OPTION = 'resource_register'
     MYPROXY_CLIENT_KEY_OPTION = 'myproxy_client_key'
     MYPROXY_GLOBAL_PASSWORD_OPTION = 'myproxy_global_password'
     USER_IDENTIFIER_KEY_OPTION = 'user_identifier_key'
@@ -79,6 +85,7 @@ class Oauth2ServerMiddleware(object):
         CLIENT_AUTHENTICATION_METHOD_OPTION: 'certificate',
         CLIENT_AUTHORIZATION_URL_OPTION: '/client_authorization/authorize',
         CLIENT_AUTHORIZATIONS_KEY_OPTION: 'client_authorizations',
+        RESOURCE_AUTHENTICATION_METHOD_OPTION: 'none',
         MYPROXY_CLIENT_KEY_OPTION: \
         'myproxy.server.wsgi.middleware.MyProxyClientMiddleware.myProxyClient',
         USER_IDENTIFIER_KEY_OPTION: 'REMOTE_USER'
@@ -144,19 +151,20 @@ class Oauth2ServerMiddleware(object):
 
         # Determine client authentication type. A 'none' options is allowed so
         # that development/testing can be performed without running on Apache.
-        if self.client_authentication_method == 'certificate':
-            client_authenticator = CertificateClientAuthenticator()
-            
-        elif self.client_authentication_method == 'none':
-            client_authenticator = NoopClientAuthenticator()
-            
-        else:
-            raise ValueError("Invalid configuration value %s for %s" %
-                             (self.client_authentication_method,
-                              self.CLIENT_AUTHENTICATION_METHOD_OPTION))
+        client_register = ClientRegister(self.client_register_file)
+        client_authenticator = self._get_authenticator(
+            self.client_authentication_method, client_register,
+            'client', self.CLIENT_AUTHENTICATION_METHOD_OPTION)
+
+        # same for resource authentication type.
+        resource_register = ResourceRegister(self.resource_register_file)
+        resource_authenticator = self._get_authenticator(
+            self.resource_authentication_method, resource_register,
+            'resource', self.RESOURCE_AUTHENTICATION_METHOD_OPTION)
 
         self._authorizationServer = AuthorizationServer(
-            self.client_register_file, authorizer, client_authenticator,
+            client_register, authorizer, client_authenticator,
+            resource_register, resource_authenticator,
             access_token_generator, conf)
 
         self._myproxy_cert_request = MyproxyCertRequest(
@@ -165,6 +173,21 @@ class Oauth2ServerMiddleware(object):
             myproxy_global_password=self.myproxy_global_password,
             user_identifier_grant_data_key=self.USER_IDENTIFIER_GRANT_DATA_KEY)
 
+    def _get_authenticator(self, name, register, typ, option_name):
+        """Returns new authenticator by name"""
+        if name == 'certificate':
+            return CertificateAuthenticator(typ, register)
+            
+        elif name == 'password':
+            return PasswordAuthenticator(typ, register)
+            
+        elif name == 'none':
+            return NoopAuthenticator(typ)
+            
+        else:
+            raise ValueError("Invalid configuration value %s for %s" %
+                             (name, option_name))
+
     def __call__(self, environ, start_response):
         """
         @type environ: dict
@@ -191,14 +214,17 @@ class Oauth2ServerMiddleware(object):
         actionPath = None
         if req.path_info.startswith(self.base_path):
             actionPath = req.path_info[len(self.base_path):]
+
         methodName = self.__class__.method.get(actionPath, '')
         if methodName:
             log.debug("Method: %s" % methodName)
             action = getattr(self, methodName)
             return action(req, start_response)
+
         elif self._app is not None:
             log.debug("Delegating to lower filter/application.")
             return self._app(environ, start_response)
+
         else:
             response = "OAuth 2.0 Server - Invalid URL"
             start_response(self._get_http_status_string(httplib.NOT_FOUND),
@@ -219,6 +245,7 @@ class Oauth2ServerMiddleware(object):
         @return: WSGI response
         """
         log.debug("authorize called")
+
         # Stop immediately if the client is not registered.
         (error, error_description
                         ) = self._authorizationServer.is_registered_client(req)
@@ -254,15 +281,17 @@ class Oauth2ServerMiddleware(object):
                                                             req, 
                                                             client_authorized)
         if error:
-            self._error_response(error, error_description, start_response)
+            return self._error_response(error, 
+                                        error_description, 
+                                        start_response)
         else:
             return self._redirect(redirect_uri, start_response)
 
     def _error_response(self, error, error_description, start_response):
         """Returns and error response.
         """
-        response = ("%s: %s" %(error, error_description)).encode('ascii',
-                                                                 'ignore')
+        response = ("%s: %s" % (error, error_description)).encode('ascii',
+                                                                  'ignore')
         log.error("Returning error: %s - %s", error, error_description)
         start_response(self._get_http_status_string(httplib.BAD_REQUEST),
                        [('Content-type', 'text/plain'),
@@ -491,6 +520,10 @@ class Oauth2ServerMiddleware(object):
                                 conf, cls.CLIENT_AUTHORIZATIONS_KEY_OPTION)
         self.client_register_file = cls._get_config_option(
                                 conf, cls.CLIENT_REGISTER_OPTION)
+        self.resource_authentication_method = cls._get_config_option(
+                                conf, cls.RESOURCE_AUTHENTICATION_METHOD_OPTION)
+        self.resource_register_file = cls._get_config_option(
+                                conf, cls.RESOURCE_REGISTER_OPTION)
         self.myproxy_client_env_key = cls._get_config_option(
                                 conf, cls.MYPROXY_CLIENT_KEY_OPTION)
         self.myproxy_global_password = cls._get_config_option(
@@ -515,3 +548,4 @@ class Oauth2ServerMiddleware(object):
     @classmethod
     def app_factory(cls, app_conf, **local_conf):
         return cls(None, app_conf, **local_conf)
+
